!MNH_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!MNH_LIC for details. version 1.
!#####################
MODULE MODI_ADV_BOUNDARIES
!#####################
!
INTERFACE ADV_BOUNDARIES_DEVICE
  MODULE PROCEDURE ADV_BOUNDARIES_DEVICE1, ADV_BOUNDARIES_DEVICE2, ADV_BOUNDARIES_DEVICE3
END INTERFACE
!
!INTERFACE
!
#if 0
      SUBROUTINE ADV_BOUNDARIES_DEVICE1 ( HLBCX,HLBCY,PFIELD )
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD
!$acc declare present(PFIELD)
!
END SUBROUTINE ADV_BOUNDARIES_DEVICE1
!
      SUBROUTINE ADV_BOUNDARIES_DEVICE2 ( HLBCX,HLBCY,PFIELD,PFIELDI )
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD
REAL, DIMENSION(:,:,:),   INTENT(IN)            :: PFIELDI
!$acc declare present(PFIELD,PFIELDI)
!
END SUBROUTINE ADV_BOUNDARIES_DEVICE2
!
      SUBROUTINE ADV_BOUNDARIES_DEVICE3 ( HLBCX,HLBCY,PFIELD,PFIELDI,HFIElD )
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD
REAL, DIMENSION(:,:,:),   INTENT(IN)            :: PFIELDI
CHARACTER(LEN=1),         INTENT(IN)            :: HFIELD  ! Field type
!$acc declare present(PFIELD,PFIELDI)
!
END SUBROUTINE ADV_BOUNDARIES_DEVICE3
!
      SUBROUTINE ADV_BOUNDARIES ( HLBCX,HLBCY,PFIELD,PFIELDI,HFIELD )
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD
REAL, DIMENSION(:,:,:),   INTENT(IN), OPTIONAL  :: PFIELDI
CHARACTER(LEN=1),         INTENT(IN), OPTIONAL  :: HFIELD  ! Field type
!
END SUBROUTINE ADV_BOUNDARIES
!
END INTERFACE
!
END MODULE MODI_ADV_BOUNDARIES
#endif
!
CONTAINS
!
!     ####################################################################
      SUBROUTINE ADV_BOUNDARIES_DEVICE1 ( HLBCX,HLBCY,PFIELD )
!     ####################################################################
!
!!****  *ADV_BOUNDARIES* - routine to prepare the top and bottom Boundary Conditions
!!
!!
!!    AUTHOR
!!    ------
!!
!!
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!
USE MODD_PARAMETERS
USE MODE_ll
!
IMPLICIT NONE
!
!
!*       0.1   declarations of arguments
!
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD
!$acc declare present(PFIELD)
!
!
!*       0.2   declarations of local variables
!
INTEGER             :: IKB       ! indice K Beginning in z direction
INTEGER             :: IKE       ! indice K End       in z direction
INTEGER             :: IIU, IJU  ! Index End in X and Y directions
!
!-------------------------------------------------------------------------------
!
!*       1.    COMPUTE DIMENSIONS OF ARRAYS AND OTHER INDICES:
!              ----------------------------------------------
IKB = 1 + JPVEXT
IKE = SIZE(PFIELD,3) - JPVEXT
IIU=SIZE(PFIELD,1)
IJU=SIZE(PFIELD,2)
!
IF (SIZE(PFIELD)==0) RETURN
!
!
!-------------------------------------------------------------------------------
!
!*       2.    UPPER AND LOWER BC FILLING:
!              ---------------------------
!
!*       2.1    COMPUTE THE FIELD EXTRAPOLATIONS AT THE GROUND
!
!$acc kernels
   PFIELD  (:,:,IKB-1)   = PFIELD  (:,:,IKB)
!
!*       2.2    COMPUTE THE FIELD EXTRAPOLATIONS AT THE TOP
!
  PFIELD  (:,:,IKE+1)   = PFIELD  (:,:,IKE)
!$acc end kernels
!
!Not enough? !$acc update self(PFIELD(:,:,IKB-1))
!Not enough? !$acc update self(PFIELD(:,:,IKE+1))
!$acc update self(PFIELD(:,:,:))
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE ADV_BOUNDARIES_DEVICE1
!
!     ####################################################################
      SUBROUTINE ADV_BOUNDARIES_DEVICE2 ( HLBCX,HLBCY,PFIELD,PFIELDI )
!     ####################################################################
!
!!****  *ADV_BOUNDARIES* - routine to prepare the top and bottom Boundary Conditions
!!
!!
!!    AUTHOR
!!    ------
!!
!!
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!
USE MODD_PARAMETERS
USE MODE_ll
!
IMPLICIT NONE
!
!
!*       0.1   declarations of arguments
!
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD
REAL, DIMENSION(:,:,:),   INTENT(IN)            :: PFIELDI
!$acc declare present(PFIELD,PFIELDI)
!
!
!*       0.2   declarations of local variables
!
INTEGER             :: IKB       ! indice K Beginning in z direction
INTEGER             :: IKE       ! indice K End       in z direction
INTEGER             :: IIU, IJU  ! Index End in X and Y directions
!
!-------------------------------------------------------------------------------
!
!*       1.    COMPUTE DIMENSIONS OF ARRAYS AND OTHER INDICES:
!              ----------------------------------------------
IKB = 1 + JPVEXT
IKE = SIZE(PFIELD,3) - JPVEXT
IIU=SIZE(PFIELD,1)
IJU=SIZE(PFIELD,2)
!
IF (SIZE(PFIELD)==0) RETURN
!
!$acc kernels
!
!-------------------------------------------------------------------------------
!
!*       2.    UPPER AND LOWER BC FILLING:
!              ---------------------------
!
!*       2.1    COMPUTE THE FIELD EXTRAPOLATIONS AT THE GROUND
!
!
   PFIELD  (:,:,IKB-1)   = PFIELD  (:,:,IKB)

!
!*       2.2    COMPUTE THE FIELD EXTRAPOLATIONS AT THE TOP
!
  PFIELD  (:,:,IKE+1)   = PFIELD  (:,:,IKE)
!
!
!*       3.    LATERAL BC FILLING
!              ---------------------------
!
  IF (HLBCX(1)=='OPEN' .AND. LWEST_ll()) THEN
     PFIELD(1,:,:) = PFIELDI(1,:,:)
  END IF
  IF (HLBCX(2)=='OPEN' .AND. LEAST_ll()) THEN
     PFIELD(IIU,:,:) = PFIELDI(IIU,:,:)
  END IF
  IF (HLBCY(1)=='OPEN' .AND. LSOUTH_ll()) THEN
     PFIELD(:,1,:) = PFIELDI(:,1,:)
  END IF
  IF (HLBCY(2)=='OPEN' .AND. LNORTH_ll()) THEN
     PFIELD(:,IJU,:) = PFIELDI(:,IJU,:)
  END IF
!$acc end kernels
!
#if 0
!Not enough?
  !$acc update self(PFIELD(:,:,IKB-1))
  !$acc update self(PFIELD(:,:,IKE+1))
  IF (HLBCX(1)=='OPEN' .AND. LWEST_ll()) THEN
    !$acc update self(PFIELD(1,:,:))
  END IF
  IF (HLBCX(2)=='OPEN' .AND. LEAST_ll()) THEN
    !$acc update self(PFIELD(IIU,:,:))
  END IF
  IF (HLBCY(1)=='OPEN' .AND. LSOUTH_ll()) THEN
    !$acc update self(PFIELD(:,1,:))
  END IF
  IF (HLBCY(2)=='OPEN' .AND. LNORTH_ll()) THEN
    !$acc update self(PFIELD(:,IJU,:))
  END IF
#else
!$acc update self(PFIELD(:,:,:))
#endif
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE ADV_BOUNDARIES_DEVICE2
!
!     ####################################################################
      SUBROUTINE ADV_BOUNDARIES_DEVICE3 ( HLBCX,HLBCY,PFIELD,PFIELDI,HFIELD )
!     ####################################################################
!
!!****  *ADV_BOUNDARIES* - routine to prepare the top and bottom Boundary Conditions
!!
!!
!!    AUTHOR
!!    ------
!!
!!
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!
USE MODD_PARAMETERS
USE MODE_ll
!
IMPLICIT NONE
!
!
!*       0.1   declarations of arguments
!
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD
REAL, DIMENSION(:,:,:),   INTENT(IN)            :: PFIELDI
!$acc declare present(PFIELD,PFIELDI)
CHARACTER(LEN=1),         INTENT(IN)            :: HFIELD  ! Field type
!
!
!*       0.2   declarations of local variables
!
INTEGER             :: IKB       ! indice K Beginning in z direction
INTEGER             :: IKE       ! indice K End       in z direction
INTEGER             :: IIU, IJU  ! Index End in X and Y directions
!
!-------------------------------------------------------------------------------
!
!*       1.    COMPUTE DIMENSIONS OF ARRAYS AND OTHER INDICES:
!              ----------------------------------------------
IKB = 1 + JPVEXT
IKE = SIZE(PFIELD,3) - JPVEXT
IIU=SIZE(PFIELD,1)
IJU=SIZE(PFIELD,2)
!
IF (SIZE(PFIELD)==0) RETURN
!
!$acc kernels
!
!-------------------------------------------------------------------------------
!
!*       2.    UPPER AND LOWER BC FILLING:
!              ---------------------------
!
!*       2.1    COMPUTE THE FIELD EXTRAPOLATIONS AT THE GROUND
!
!
   IF (HFIELD=='W') &
   PFIELD  (:,:,IKB  )   = PFIELDI (:,:,IKB)
!
   PFIELD  (:,:,IKB-1)   = PFIELD  (:,:,IKB)

!
!*       2.2    COMPUTE THE FIELD EXTRAPOLATIONS AT THE TOP
!
  PFIELD  (:,:,IKE+1)   = PFIELD  (:,:,IKE)
!
!
!*       3.    LATERAL BC FILLING
!              ---------------------------
!
  IF (HLBCX(1)=='OPEN' .AND. LWEST_ll()) THEN
     PFIELD(1,:,:) = PFIELDI(1,:,:)
     IF (HFIELD=='U') &
     PFIELD(2,:,:) = PFIELDI(2,:,:)
  END IF
  IF (HLBCX(2)=='OPEN' .AND. LEAST_ll()) THEN
     PFIELD(IIU,:,:) = PFIELDI(IIU,:,:)
  END IF
  IF (HLBCY(1)=='OPEN' .AND. LSOUTH_ll()) THEN
     PFIELD(:,1,:) = PFIELDI(:,1,:)
     IF (HFIELD=='V') &
     PFIELD(:,2,:) = PFIELDI(:,2,:)
  END IF
  IF (HLBCY(2)=='OPEN' .AND. LNORTH_ll()) THEN
     PFIELD(:,IJU,:) = PFIELDI(:,IJU,:)
  END IF
!$acc end kernels
!
#if 0
!Not enough?
!add also if hfield =u or v
  !$acc update self(PFIELD(:,:,IKB-1))
  !$acc update self(PFIELD(:,:,IKE+1))
  IF (HLBCX(1)=='OPEN' .AND. LWEST_ll()) THEN
    !$acc update self(PFIELD(1,:,:))
  END IF
  IF (HLBCX(2)=='OPEN' .AND. LEAST_ll()) THEN
    !$acc update self(PFIELD(IIU,:,:))
  END IF
  IF (HLBCY(1)=='OPEN' .AND. LSOUTH_ll()) THEN
    !$acc update self(PFIELD(:,1,:))
  END IF
  IF (HLBCY(2)=='OPEN' .AND. LNORTH_ll()) THEN
    !$acc update self(PFIELD(:,IJU,:))
  END IF
#else
!$acc update self(PFIELD(:,:,:))
#endif
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE ADV_BOUNDARIES_DEVICE3
!
!     ####################################################################
      SUBROUTINE ADV_BOUNDARIES ( HLBCX,HLBCY,PFIELD,PFIELDI,HFIELD )
!     ####################################################################
!
!!****  *ADV_BOUNDARIES* - routine to prepare the top and bottom Boundary Conditions 
!!
!!
!!    AUTHOR
!!    ------
!!	
!!
!-------------------------------------------------------------------------------
!
!*       0.    DECLARATIONS
!         
USE MODD_PARAMETERS
USE MODE_ll
!
IMPLICIT NONE
!
!
!*       0.1   declarations of arguments
!
!
CHARACTER(LEN=4), DIMENSION(2), INTENT(IN)      :: HLBCX,HLBCY   ! X and Y-direc. LBC type
REAL, DIMENSION(:,:,:),   INTENT(INOUT)         :: PFIELD                        
REAL, DIMENSION(:,:,:),   INTENT(IN), OPTIONAL  :: PFIELDI                        
CHARACTER(LEN=1),         INTENT(IN), OPTIONAL  :: HFIELD  ! Field type
!
!
!*       0.2   declarations of local variables
!
INTEGER             :: IKB       ! indice K Beginning in z direction
INTEGER             :: IKE       ! indice K End       in z direction 
INTEGER             :: IIU, IJU  ! Index End in X and Y directions
!
!-------------------------------------------------------------------------------
!
!*       1.    COMPUTE DIMENSIONS OF ARRAYS AND OTHER INDICES:
!              ----------------------------------------------
IKB = 1 + JPVEXT
IKE = SIZE(PFIELD,3) - JPVEXT
IIU=SIZE(PFIELD,1)
IJU=SIZE(PFIELD,2)
!
IF (SIZE(PFIELD)==0) RETURN
!-------------------------------------------------------------------------------
!
!*       2.    UPPER AND LOWER BC FILLING:   
!              ---------------------------
!
!*       2.1    COMPUTE THE FIELD EXTRAPOLATIONS AT THE GROUND
!
!
   IF (PRESENT(HFIELD) .AND. PRESENT(PFIELDI)) THEN
     IF (HFIELD=='W') &
     PFIELD  (:,:,IKB  )   = PFIELDI (:,:,IKB) 
   END IF
!
   PFIELD  (:,:,IKB-1)   = PFIELD  (:,:,IKB) 

!
!*       2.2    COMPUTE THE FIELD EXTRAPOLATIONS AT THE TOP
!
  PFIELD  (:,:,IKE+1)   = PFIELD  (:,:,IKE) 
!
!
!*       3.    LATERAL BC FILLING                                
!              ---------------------------
!
IF( PRESENT(PFIELDI) )  THEN
  IF (HLBCX(1)=='OPEN' .AND. LWEST_ll()) THEN
     PFIELD(1,:,:) = PFIELDI(1,:,:)
     IF (PRESENT(HFIELD)) THEN
       IF (HFIELD=='U') &
       PFIELD(2,:,:) = PFIELDI(2,:,:)
     END IF
  END IF
  IF (HLBCX(2)=='OPEN' .AND. LEAST_ll()) THEN
     PFIELD(IIU,:,:) = PFIELDI(IIU,:,:)
  END IF
  IF (HLBCY(1)=='OPEN' .AND. LSOUTH_ll()) THEN
     PFIELD(:,1,:) = PFIELDI(:,1,:)
     IF (PRESENT(HFIELD)) THEN
       IF (HFIELD=='V') &
       PFIELD(:,2,:) = PFIELDI(:,2,:)
     END IF
  END IF
  IF (HLBCY(2)=='OPEN' .AND. LNORTH_ll()) THEN
     PFIELD(:,IJU,:) = PFIELDI(:,IJU,:)
  END IF
END IF
!
!-------------------------------------------------------------------------------
!
END SUBROUTINE ADV_BOUNDARIES

END MODULE MODI_ADV_BOUNDARIES
